//
//  SFICOFamily.h
//  SFToolkit
//
//  Created by Konstantin Erokhin on 5/5/11.
//  Copyright 2011 Shiny Frog. All rights reserved.
//

#import <Foundation/Foundation.h>

enum {
	SFICOFamilyElement256	= kIconServices256PixelDataARGB,
	SFICOFamilyElement48	= kIconServices48PixelDataARGB,
	SFICOFamilyElement32	= kIconServices32PixelDataARGB,
	SFICOFamilyElement16	= kIconServices16PixelDataARGB
};
typedef OSType SFICOFamilyElement;

@interface SFICOFamily : NSObject {
@private
    NSData * mIconFamilyData;
	CGImageSourceRef mImageSourceRef;
}

// -------------------------------------------------------------------------------
//	Creation
// -------------------------------------------------------------------------------

+ (SFICOFamily *)iconFamily;
+ (SFICOFamily *)iconFamilyWithContentsOfURL:(NSURL *)url;
+ (SFICOFamily *)iconFamilyWithIconOfURL:(NSURL *)url;
+ (SFICOFamily *)iconFamilyWithThumbnailsOfImage:(NSImage *)image imageInterpolation:(NSImageInterpolation)imageInterpolation;

- (id)initWithContentsOfURL:(NSURL *)url;
- (id)initWithIconOfURL:(NSURL *)url;

// Resizes the image to create each element
- (id)initWithThumbnailsOfImage:(NSImage *)image imageInterpolation:(NSImageInterpolation)imageInterpolation;

// -------------------------------------------------------------------------------
//	Image I/O
// -------------------------------------------------------------------------------

- (CGImageRef)CGImageForElement:(SFICOFamilyElement)elementType;
- (BOOL)setCGImage:(CGImageRef)imageRef forElement:(SFICOFamilyElement)elementType;
- (BOOL)setBitmapImageRep:(NSBitmapImageRep *)bitmapImageRep forElement:(SFICOFamilyElement)elementType;
- (NSImage *)imageWithAllElements;

- (NSBitmapImageRep *)bitmapImageRepForElement:(SFICOFamilyElement)elementType;
// -------------------------------------------------------------------------------
//	File I/O
// -------------------------------------------------------------------------------

- (BOOL)setAsCustomIconForURL:(NSURL *)url;
+ (BOOL)removeCustomIconFromURL:(NSURL *)url;

// Writes the icon family to an .ico file.
- (BOOL)writeToURL:(NSURL *)url error:(NSError **)error;
- (BOOL)writeToURL:(NSURL *)url onlyElementType:(SFICOFamilyElement)elementType error:(NSError **)error;

@end
